% EDNMR   Calculate EDNMR spectrum for spin system
%         with S=1/2 and one nucleus.
%
%   [nu,spec] = EDNMR(Sys,Exp)
%   ... = EDNMR(Sys,Exp,Opt)
%
%   Input:
%     Sys      spin system structure
%     Exp      experiment structure
%       Exp.Field        magnetic field, in mT
%       Exp.Range        RF range [RFmin RFmax], in MHz
%       Exp.nPoints      number of points
%       Exp.ExciteWidth  excitation bandwidth, in MHz
%       Exp.Q            Q0 of the cavity (set to 1 for no frequency dependence)
%       Exp.HTA          HTA pulse length, in s
%       Exp.w1           w1, in rad/s
%       Exp.T2           decay time of EDNMR nutations in s
%     Opt      options structure
%       Opt.Symmetry     symmetry of spin system
%       Opt.nKnots       number of knots for the orientation grid
%       Opt.Threshold    intensity cutoff

function [nu,spec] = EDNMR(Sys,Exp,Opt)

% Defaults for input parameters and fields
%-------------------------------------------------------------------------------
if nargin<3, Opt = struct; end
if ~isfield(Opt,'Symmetry'), Opt.Symmetry = 'Ci'; end
if ~isfield(Opt,'nKnots'), Opt.nKnots = 91; end
if ~isfield(Opt,'Threshold'), Opt.Threshold = 1e-4; end
if ~isfield(Exp,'Q'), Exp.Q = 1; end

% Orientational grid and orientation selection
%-------------------------------------------------------------------------------
% Orientation grid
[phi,theta,powderWeight] = sphgrid(Opt.Symmetry,Opt.nKnots);
nOrientations = numel(powderWeight);

% Orientation pre-selection
oriselWeight = orisel(Sys,Exp,Opt);

% Other preparations
%-------------------------------------------------------------------------------
% Pre-calculate spin Hamiltonian components in molecular frame
[F,GxM,GyM,GzM] = sham(Sys);
zeemanop = @(v) v(1)*GxM + v(2)*GyM + v(3)*GzM;

% Size of nuclear spin space
I = nucspin(Sys.Nucs);
nNucStates = prod(2*I+1);

if (Sys.S==1/2)
  manifold1 = 1;     % 1st manifold = alpha manifold
  manifold2 = 2;     % 2nd manifold = beta manifold
elseif (Sys.S==5/2)
  % Select central transition only
  manifold1 = 3;
  manifold2 = 4;
else
  error('Sys.S not supported.')
end

% Loop over orientations
%-------------------------------------------------------------------------------
Pos = [];
for iOri = 1:nOrientations
  % Calculate only orientations with weight above cutoff
  if oriselWeight(iOri)<=Opt.Threshold, continue; end
  
  % Set up lab axes for this orientation
  [xL,yL,zL] = erot([phi(iOri),theta(iOri),0],'rows');
  % zL = z laboratoy axis: external static field
  % xL = x laboratory axis: B1 excitation field
  % yL = y laboratory vector: needed for intensity integration over third Euler angle
  
  % Set up spin Hamiltonian and calculate eigenvalues and eigenvectors
  H0 = F + Exp.Field*zeemanop(zL);
  [V,E] = eig(H0);
  E = diag(E);
  E = repmat(E,[1,length(E)]);
  dE = abs(E-E.');      % EPR/ENDOR positions (energy differences)
  P = abs((V'*zeemanop(xL)*V).^2) + abs((V'*zeemanop(yL)*V).^2); % EPR/ENDOR intensities
  
  % Filter ELDOR positions for correct sign (relative to a given EPR transition)
  m1 = (manifold1-1)*nNucStates + (1:nNucStates);
  m2 = (manifold2-1)*nNucStates + (1:nNucStates);
  EB = [dE(m2,m1) dE(m1,m2)]; % EB = dE without NMR positions
  TB = [P(m2,m1) P(m1,m2)]; % TB = corresponding transition intensities
  
  condition = dE(2,1) > dE(nNucStates+1,nNucStates+2); % compare frequencies of mI<->mI-1 NMR transition
  sumTB = sum(TB); % normalization factor for set of transitions i.e. each column of TB
  idx = 1;
  for s1 = 1:nNucStates*2
    TB(:,s1) = TB(:,s1)/sumTB(s1);
    for s2 = 1:nNucStates
      EX = EB(s2,s1) - EB(:,s1); % EPR (probe) - ELDOR (pump) = EDNMR
      if condition
        EX = -EX;              % ELDOR (pump) -EPR (probe) = EDNMR
      end
      TX = TB(:,s1)+0.1;
      TX(s2) = 0;
      Probe(idx,1) = EB(s2,s1);   % probe/EPR frequencies
      Positions(idx,1:nNucStates-1) = nonzeros(EX);   % pump/EDNMR frequencies/filter out probe
      Probe_Int(idx,1) = TB(s2,s1); % probe/EPR transition
      Intensities(idx,1:nNucStates-1) = nonzeros(TX)-0.1; % pump/EDNMR transitions/filter out probe
      idx = idx + 1;
    end
  end
  Pos(:,iOri)= Positions(:);
  Amp(:,iOri) = Intensities(:);
  
  % Orientation selection
  for m = 1:size(Positions,2)
    Orient(:,m) = exp(-2*((Probe(:,1)-Exp.mwFreq*1e3)/Exp.ExciteWidth).^2).*Probe_Int(:,1);
  end
  excitationWeight(:,iOri) = Orient(:);
  
end

if ~any(Pos(:))
  error('No ELDOR transitions at %g mT',Exp.Field);
end

% Calculate EDNMR spectrum
%-------------------------------------------------------------------------------
for iOri = 1:nOrientations
  % Calculate only orientations with weight above cutoff
  if oriselWeight(iOri)<=Opt.Threshold, continue; end
  for iLine = 1:size(Amp,1)
    if Amp(iLine,iOri)==0, continue; end
    
    offset = Pos(iLine,iOri)*4*Exp.Q/(Exp.mwFreq*1e3);
    w1 = Exp.w1*sqrt(1/(1+offset^2/4)); % W1(HTA) at EDNMR position using experimental loaded Q-value
    an = w1*sqrt(Amp(iLine,iOri))*Exp.HTA;     % HTA efficiency w1(HTA)*sqrt(If)*t(HTA)
    an_T2 = Exp.HTA/Exp.T2;             % calculate damping factor
    
    % Bloch evolution of magnetization during HTA pulse
    % (a) slow method using numeric matrix exponential
    %M = expm([-an_T2 0 0; 0 -an_T2 -an; 0 an 0])*[0;0;1];
    %Mz = M(3);
    % (b) fast method using analytical expression
    x_ = sqrt(an_T2^2-4*an^2);
    Mz = exp(-an_T2/2)*(cosh(x_/2)+an_T2*sinh(x_/2)/x_);
    
    % Calculate amplitude of EDNMR line
    Amp(iLine,iOri) = excitationWeight(iLine,iOri)*powderWeight(iOri)*(1-Mz)/2;
  end
end

% Construct EDNMR spectrum
%-------------------------------------------------------------------------------
[nu,spec] = makespec(Exp.Range,Exp.nPoints,Pos(:),Amp(:));
dnu = nu(2)-nu(1);
spec = convspec(spec,dnu,Sys.lwEndor);
